@@ -16,8 +16,8 @@
 #' * Supply `call` to inform users about which function the error
 #'   occurred in.
 #'
-#' * Supply another condition as `parent` to create a chained
-#'   condition.
+#' * Supply another condition as `parent` to create a [chained
+#'   condition][topic-error-chaining].
 #'
 #' Certain components of condition messages are formatted with unicode
 #' symbols and terminal colours by default. These aspects can be
@@ -26,16 +26,20 @@
 #' @inheritParams cnd
 #' @param message The message to display, formatted as a __bulleted
 #'   list__. The first element is displayed as an _alert_ bullet
-#'   prefixed with `!` by default. Elements named `"*"`, `"i"`, and
-#'   `"x"` are formatted as regular, info, and cross bullets
-#'   respectively. See `r link("topic_condition_formatting")` for more
-#'   about bulleted messaging.
+#'   prefixed with `!` by default. Elements named `"*"`, `"i"`, `"v"`,
+#'   `"x"`, and `"!"` are formatted as regular, info, success,
+#'   failure, and error bullets respectively. See `r link("topic_condition_formatting")`
+#'   for more about bulleted messaging.
 #'
 #'   If a message is not supplied, it is expected that the message is
 #'   generated __lazily__ through [cnd_header()] and [cnd_body()]
 #'   methods. In that case, `class` must be supplied. Only `inform()`
 #'   allows empty messages as it is occasionally useful to build user
 #'   output incrementally.
+#'
+#'   If a function, it is stored in the `header` field of the error
+#'   condition. This acts as a [cnd_header()] method that is invoked
+#'   lazily when the error message is displayed.
 #' @param class Subclass of the condition.
 #' @param ... Additional data to be stored in the condition object.
 #'   If you supply condition fields, you should usually provide a
@@ -54,6 +58,8 @@
 #'
 #'   Can also be `NULL` or a [defused function call][topic-defuse] to
 #'   respectively not display any call or hard-code a code to display.
+#'
+#'   For more information about error calls, see `r link("topic_error_call")`.
 #' @param parent Supply `parent` when you rethrow an error from a
 #'   condition handler (e.g. with [try_fetch()]).
 #'
@@ -70,6 +76,8 @@
 #'     from a condition handler. This helps it create simpler
 #'     backtraces where the condition handling context is hidden by
 #'     default.
+#'
+#'   For more information about error calls, see `r link("topic_error_chaining")`.
 #' @param use_cli_format Whether to format `message` lazily using
 #'   [cli](https://cli.r-lib.org/) if available. This results in
 #'   prettier and more accurate formatting of messages. See
@@ -80,6 +88,11 @@
 #'   individual and unformatted lines. Any newline character `"\\n"`
 #'   already present in `message` is reformatted by cli's paragraph
 #'   formatter. See `r link("topic_condition_formatting")`.
+#' @param .inherit Whether the condition inherits from `parent`
+#'   according to [cnd_inherits()] and [try_fetch()]. By default,
+#'   parent conditions of higher severity are not inherited. For
+#'   instance an error chained to a warning is not inherited to avoid
+#'   unexpectedly catching an error downgraded to a warning.
 #' @param .internal If `TRUE`, a footer bullet is added to `message`
 #'   to let the user know that the error is internal and that they
 #'   should report it to the package authors. This argument is
@@ -87,12 +100,15 @@
 #' @param .file A connection or a string specifying where to print the
 #'   message. The default depends on the context, see the `stdout` vs
 #'   `stderr` section.
-#' @param .frame The throwing context. Defaults to `call` if it is an
-#'   environment, [caller_env()] otherwise. This is used in the
-#'   display of simplified backtraces as the last relevant call frame
-#'   to show. This way, the irrelevant parts of backtraces
-#'   corresponding to condition handling ([tryCatch()], [try_fetch()],
-#'   `abort()`, etc.) are hidden by default.
+#' @param .frame The throwing context. Used as default for
+#'   `.trace_bottom`, and to determine the internal package to mention
+#'   in internal errors when `.internal` is `TRUE`.
+#' @param .trace_bottom Used in the display of simplified backtraces
+#'   as the last relevant call frame to show. This way, the irrelevant
+#'   parts of backtraces corresponding to condition handling
+#'   ([tryCatch()], [try_fetch()], `abort()`, etc.) are hidden by
+#'   default. Defaults to `call` if it is an environment, or `.frame`
+#'   otherwise. Without effect if `trace` is supplied.
 #' @param .subclass `r lifecycle::badge("deprecated")` This argument
 #'   was renamed to `class` in rlang 0.4.2 for consistency with our
 #'   conventions for class constructors documented in
@@ -167,6 +183,10 @@
 #'   especially easy to hit when the message contains a lot of ANSI
 #'   escapes, as created by the crayon or cli packages
 #'
+#' @seealso
+#' - `r link("topic_error_call")`
+#' - `r link("topic_error_chaining")`
+#'
 #' @examples
 #' # These examples are guarded to avoid throwing errors
 #' if (FALSE) {
@@ -237,35 +257,47 @@ abort <- function(message = NULL,
                   trace = NULL,
                   parent = NULL,
                   use_cli_format = NULL,
+                  .inherit = TRUE,
                   .internal = FALSE,
                   .file = NULL,
-                  .frame = NULL,
+                  .frame = caller_env(),
+                  .trace_bottom = NULL,
                   .subclass = deprecated()) {
+  check_environment(.frame)
+
   .__signal_frame__. <- TRUE
-  caller <- caller_env()
 
   rethrowing <- !is_null(parent)
   if (is_na(parent)) {
     parent <- NULL
   }
 
+  if (is_list(maybe_missing(call))) {
+    if (!identical(names(call), c("call", "frame")) &&
+        !identical(names(call), c("", "frame"))) {
+      abort("When a list, `call` must have \"call\" and \"frame\" names.")
+    }
+    .frame <- call[["frame"]] %||% .frame
+    call <- call[["call"]]
+  }
+
   # `.frame` is used to soft-truncate the backtrace
-  if (is_null(.frame)) {
+  if (is_null(.trace_bottom)) {
     if (rethrowing) {
-      .frame <- caller
+      .trace_bottom <- .frame
     } else {
       # Truncate backtrace up to `call` if it is a frame
       if (is_environment(maybe_missing(call))) {
-        .frame <- call
+        .trace_bottom <- call
       } else {
-        .frame <- caller
+        .trace_bottom <- .frame
       }
     }
   } else {
-    check_environment(.frame)
+    check_environment(.trace_bottom)
   }
 
-  info <- abort_context(.frame, rethrowing)
+  info <- abort_context(.trace_bottom, rethrowing, maybe_missing(call))
 
   if (is_missing(call)) {
     if (is_null(info$from_handler)) {
@@ -277,14 +309,18 @@ abort <- function(message = NULL,
     call <- info$setup_caller
   }
 
+  if (is_formula(message, scoped = TRUE, lhs = FALSE)) {
+    message <- as_function(message)
+  }
+
   message <- validate_signal_args(message, class, call, .subclass, "abort")
-  call <- error_call(call)
+  error_call <- error_call(call)
 
   message_info <- cnd_message_info(
     message,
     body,
     footer,
-    caller,
+    .frame,
     use_cli_format = use_cli_format,
     internal = .internal
   )
@@ -292,13 +328,30 @@ abort <- function(message = NULL,
   extra_fields <- message_info$extra_fields
   use_cli_format <- message_info$use_cli_format
 
-  if (is_null(trace) && is_null(peek_option("rlang:::disable_trace_capture"))) {
-    with_options(
-      # Prevents infloops when rlang throws during trace capture
-      "rlang:::disable_trace_capture" = TRUE,
-      "rlang:::visible_bottom" = info$bottom_frame,
-      { trace <- trace_back() }
-    )
+  extra_fields$rlang <- c(
+    extra_fields$rlang,
+    list(inherit = .inherit)
+  )
+
+  parent_trace <- if (rethrowing) parent[["trace"]]
+
+  if (!is_null(parent_trace) && is_environment(call)) {
+    calls <- sys.calls()
+    frames <- sys.frames()
+
+    loc_frame <- detect_index(frames, identical, call, .right = TRUE)
+    if (loc_frame && loc_frame <= nrow(parent_trace)) {
+      parent_call <- parent_trace[["call"]][[loc_frame]]
+      this_call <- frame_call(call)
+
+      if (identical(parent_call, this_call)) {
+        if (is_null(parent_trace[["error_frame"]])) {
+          parent_trace[["error_frame"]] <- FALSE
+        }
+        parent_trace[["error_frame"]][[loc_frame]] <- TRUE
+        parent$trace <- parent_trace
+      }
+    }
   }
 
   cnd <- error_cnd(
@@ -307,14 +360,29 @@ abort <- function(message = NULL,
     message = message,
     !!!extra_fields,
     use_cli_format = use_cli_format,
-    call = call,
-    parent = parent,
-    trace = trace
+    call = error_call,
+    parent = parent
   )
+
+  if (is_null(trace) && is_null(parent_trace) && is_null(peek_option("rlang:::disable_trace_capture"))) {
+    with_options(
+      # Prevents infloops when rlang throws during trace capture
+      "rlang:::disable_trace_capture" = TRUE,
+      "rlang:::visible_bottom" = info$bottom_frame,
+      "rlang:::error_frame" = if (is_environment(call)) call else NULL,
+      "rlang:::error_arg" = cnd[["arg"]],
+      { trace <- trace_back() }
+    )
+  }
+  cnd$trace <- trace
+
   signal_abort(cnd, .file)
 }
 
-abort_context <- function(frame, rethrowing, call = caller_env()) {
+abort_context <- function(frame,
+                          rethrowing,
+                          abort_call,
+                          call = caller_env()) {
   calls <- sys.calls()
   frames <- sys.frames()
   parents <- sys.parents()
@@ -384,11 +452,17 @@ abort_context <- function(frame, rethrowing, call = caller_env()) {
     # Skip frames marked with the sentinel `.__signal_frame__.`
     bottom_loc <- skip_signal_frames(bottom_loc, frames)
     bottom_frame <- frames[[bottom_loc]]
+    if (!rethrowing && !is_missing(abort_call) && is_environment(abort_call)) {
+      abort_call_loc <- detect_index(frames, identical, abort_call)
+      if (abort_call_loc && abort_call_loc < bottom_loc) {
+        bottom_frame <- frames[[abort_call_loc]]
+      }
+    }
   } else {
     bottom_frame <- NULL
   }
 
-  if (is_null(setup_caller) && setup_loc) {
+  if (is_null(setup_caller) && setup_loc && parents[[setup_loc]]) {
     setup_caller <- frames[[parents[[setup_loc]]]]
   }
 
@@ -503,7 +577,7 @@ skip_signal_frames <- function(loc, frames) {
 }
 
 is_calling_handler_inlined_call <- function(call) {
-  is_call(call) && is_function(call[[1]]) && is_condition(call[[2]])
+  is_call(call) && length(call) >= 2 && is_function(call[[1]]) && is_condition(call[[2]])
 }
 is_calling_handler_simple_error_call <- function(call1, call2) {
   identical(call1, quote(h(simpleError(msg, call)))) && is_call(call2, ".handleSimpleError")
@@ -524,6 +598,13 @@ cnd_message_info <- function(message,
     check_exclusive(footer, .internal, .require = FALSE, .frame = error_call)
   }
 
+  if (is_function(message)) {
+    header <- message
+    message <- ""
+  } else {
+    header <- NULL
+  }
+
   if (length(message) > 1 && !is_character(body) && !is_null(body)) {
     stop_multiple_body(body, call = error_call)
   }
@@ -551,11 +632,14 @@ cnd_message_info <- function(message,
     } else {
       fields$body <- body
     }
+    if (!is_null(header)) {
+      fields$header <- header
+    }
     if (!is_null(footer)) {
       fields$footer <- footer
     }
     if (internal) {
-      fields$footer <- footer_internal
+      fields$footer <- footer_internal(env)
     }
   } else {
     # Compatibility with older bullets formatting
@@ -573,9 +657,13 @@ cnd_message_info <- function(message,
       fields$footer <- footer
     }
     if (internal) {
-      message <- c(message, footer_internal)
+      message <- c(message, footer_internal(env))
     }
     message <- .rlang_cli_format_fallback(message)
+
+    if (is_function(header)) {
+      fields$header <- header
+    }
   }
 
   list(
@@ -586,9 +674,31 @@ cnd_message_info <- function(message,
 }
 utils::globalVariables(".internal")
 
-footer_internal <- c(
-  "i" = "This is an internal error, please report it to the package authors."
-)
+footer_internal <- function(env) {
+  top <- topenv(env)
+  url_line <- NULL
+
+  if (is_namespace(top)) {
+    pkg <- ns_env_name(top)
+    pkg_line <- sprintf(
+      "This is an internal error that was detected in the %s package.",
+      format_pkg(pkg)
+    )
+
+    url <- pkg_url_bug(pkg)
+    if (!is_null(url)) {
+      url_line <- sprintf(
+        "Please report it at %s with a %s and the full backtrace.",
+        format_url(url),
+        format_href("reprex", "https://tidyverse.org/help/")
+      )
+    }
+  } else {
+    pkg_line <- "This is an internal error, please report it to the package authors."
+  }
+
+  c("i" = pkg_line, " " = url_line)
+}
 
 stop_multiple_body <- function(body, call) {
   msg <- c(
@@ -605,7 +715,7 @@ stop_multiple_body <- function(body, call) {
     "i" = sprintf(
       "%s is currently %s.",
       format_arg("body"),
-      friendly_type_of(body)
+      obj_type_friendly(body)
     )
   )
   abort(msg, call = call)
@@ -708,7 +818,7 @@ check_use_cli_flag <- function(flag, error_call) {
   }
 
   if (flag[["inline"]]) {
-    if (!has_cli_format || !has_cli_inline) {
+    if (!has_cli_format) {
       msg <- c(
         "`.__rlang_use_cli__.[[\"inline\"]]` is set to `TRUE` but cli is not installed or is too old.",
         "i" = "The package author should add a recent version of `cli` to their `Imports`."
@@ -751,15 +861,17 @@ signal_abort <- function(cnd, file = NULL) {
   # Save the unhandled error for `rlang::last_error()`.
   poke_last_error(cnd)
 
-  # Include backtrace footer option in the condition
-  cnd <- cnd_set_backtrace_on_error(cnd, peek_backtrace_on_error())
+  if (peek_show_error_messages()) {
+    # Include backtrace footer option in the condition
+    cnd <- cnd_set_backtrace_on_error(cnd, peek_backtrace_on_error())
 
-  # Print the error manually. This allows us to use our own style,
-  # include parent errors, and work around limitations on the length
-  # of error messages (#856).
-  msg <- cnd_message(cnd, inherit = TRUE, prefix = TRUE)
+    # Print the error manually. This allows us to use our own style,
+    # include parent errors, and work around limitations on the length
+    # of error messages (#856).
+    msg <- cnd_message(cnd, inherit = TRUE, prefix = TRUE)
 
-  cat_line(msg, file = file %||% default_message_file())
+    cat_line(msg, file = file %||% default_message_file())
+  }
 
   # Use `stop()` to run the `getOption("error")` handler (used by
   # RStudio to record a backtrace) and cause a long jump. Running the
@@ -770,6 +882,13 @@ signal_abort <- function(cnd, file = NULL) {
   stop(fallback)
 }
 
+peek_show_error_messages <- function() {
+  # `abort()` respects the base R option `show.error.messages` (#1630).
+  # The only time we don't display error messages is an explicit `FALSE`.
+  # All other values still show error messages.
+  !is_false(peek_option("show.error.messages"))
+}
+
 #' Set local error call in an execution environment
 #'
 #' `local_error_call()` is an alternative to explicitly passing a
@@ -936,10 +1055,17 @@ local_error_call <- function(call, frame = caller_env()) {
 #'   field named `call`. An easy way to do this is by passing a `call`
 #'   argument to [abort()]. See also [local_error_call()].
 #'
-#' @param arg,error_arg An argument name as a string. This argument
+#' @param arg An argument name as a string. This argument
 #'   will be mentioned in error messages as the input that is at the
 #'   origin of a problem.
-#' @param call,error_call The execution environment of a currently
+#' @param error_arg An argument name as a string. This argument
+#'   will be mentioned in error messages as the input that is at the
+#'   origin of a problem.
+#' @param call The execution environment of a currently
+#'   running function, e.g. `caller_env()`. The function will be
+#'   mentioned in error messages as the source of the error. See the
+#'   `call` argument of [rlang::abort()] for more information.
+#' @param error_call The execution environment of a currently
 #'   running function, e.g. `caller_env()`. The function will be
 #'   mentioned in error messages as the source of the error. See the
 #'   `call` argument of [rlang::abort()] for more information.
@@ -968,7 +1094,7 @@ NULL
 #'
 #' @examples
 #' arg_checker <- function(x, arg = caller_arg(x), call = caller_env()) {
-#'   cli::cli_abort("{.arg arg} must be a thingy.", call = call)
+#'   cli::cli_abort("{.arg {arg}} must be a thingy.", arg = arg, call = call)
 #' }
 #'
 #' my_function <- function(my_arg) {
@@ -989,13 +1115,18 @@ caller_arg <- function(arg) {
 #'
 #' @description
 #'
-#' `format_error_call()` simplifies its input to a simple call (see
-#' section below) and formats the result as code (using cli if
-#' available). Use this function to generate the "in" part
-#' of an error message from a stack frame call.
+#' - `error_call()` takes either a frame environment or a call. If the
+#'   input is an environment, `error_call()` acts like [frame_call()]
+#'   with some additional logic, e.g. for S3 methods and for frames
+#'   with a [local_error_call()].
 #'
-#' If passed an environment, the corresponding `sys.call()` is taken
-#' as call, unless there is a local flag (see [local_error_call()]).
+#' - `format_error_call()` simplifies its input to a simple call (see
+#'   section below) and formats the result as code (using cli if
+#'   available). Use this function to generate the "in" part of an
+#'   error message from a stack frame call.
+#'
+#'   `format_error_call()` first passes its input to `error_call()` to
+#'   fetch calls from frame environments.
 #'
 #' @section Details of formatting:
 #'
@@ -1040,10 +1171,10 @@ format_error_call <- function(call) {
   }
 
   if (grepl("\n", label)) {
-    cli_with_whiteline_escapes(label, format_code)
-  } else {
-    format_code(label)
+    return(cli_with_whiteline_escapes(label, format_code))
   }
+
+  format_code(label)
 }
 
 error_call_as_string <- function(call) {
@@ -1078,7 +1209,11 @@ error_call_as_string <- function(call) {
   }
 
   if (!is_call_simple(call)) {
-    return(NULL)
+    if (is_expression(call) && is_call_index(call)) {
+      return(as_label(call[1]))
+    } else {
+      return(NULL)
+    }
   }
 
   # Remove namespace for now to simplify conversion
@@ -1117,6 +1252,8 @@ error_call_deparse <- function(call) {
   out
 }
 
+#' @rdname format_error_call
+#' @export
 error_call <- function(call) {
   while (is_environment(call)) {
     flag <- env_get(call, ".__error_call__.", default = TRUE)
@@ -1188,7 +1325,6 @@ call_restore <- function(x, to) {
 #' * `"reminder"`, the default in interactive sessions, displays a reminder that
 #'   you can see the backtrace with [rlang::last_error()].
 #' * `"branch"` displays a simplified backtrace.
-#' * `"collapse"` displays a collapsed backtrace tree.
 #' * `"full"`, the default in non-interactive sessions, displays the full tree.
 #'
 #' rlang errors are normally thrown with [abort()]. If you promote
@@ -1206,7 +1342,7 @@ call_restore <- function(x, to) {
 #' * It prints the backtrace for the current error according to the
 #'   `rlang_backtrace_on_error` option.
 #'
-#' @section Errors in RMarkdown:
+#' @section Warnings and errors in RMarkdown:
 #'
 #' The display of errors depends on whether they're expected (i.e.
 #' chunk option `error = TRUE`) or unexpected:
@@ -1234,8 +1370,29 @@ call_restore <- function(x, to) {
 #' backtraces. If needed, you can override this by setting the
 #' `rlang_trace_top_env` global option.
 #'
+#' Similarly to `rlang_backtrace_on_error_report`, you can set
+#' `rlang_backtrace_on_warning_report` inside RMarkdown documents to
+#' tweak the display of warnings. This is useful in conjunction with
+#' [global_entrace()]. Because of technical limitations, there is
+#' currently no corresponding `rlang_backtrace_on_warning` option for
+#' normal R sessions.
+#'
+#' To get full entracing in an Rmd document, include this in a setup
+#' chunk before the first error or warning is signalled.
+#'
+#' ````
+#' ```{r setup}
+#' rlang::global_entrace()
+#' options(rlang_backtrace_on_warning_report = "full")
+#' options(rlang_backtrace_on_error_report = "full")
+#' ```
+#' ````
+#'
+#'
 #' @name rlang_backtrace_on_error
-#' @aliases add_backtrace
+#' @seealso rlang_backtrace_on_warning
+#' @aliases add_backtrace rlang_backtrace_on_error_report
+#'   rlang_backtrace_on_warning_report
 #'
 #' @examples
 #' # Display a simplified backtrace on error for both base and rlang
@@ -1248,7 +1405,7 @@ call_restore <- function(x, to) {
 #' # stop("foo")
 NULL
 
-backtrace_on_error_opts <- c("none", "reminder", "branch", "collapse", "full")
+backtrace_on_error_opts <- c("none", "reminder", "branch", "full")
 
 # Whenever the backtrace-on-error format is changed, the version in
 # `inst/backtrace-ver` and in `tests/testthat/helper-rlang.R` must be
@@ -1279,7 +1436,8 @@ format_onerror_backtrace <- function(cnd, opt = peek_backtrace_on_error()) {
   # reminder when there is no trace to display
   if (opt == "reminder") {
     if (is_interactive()) {
-      reminder <- silver("Run `rlang::last_error()` to see where the error occurred.")
+      last_error <- style_rlang_run("last_trace()")
+      reminder <- col_silver(paste0("Run `", last_error, "` to see where the error occurred."))
     } else {
       reminder <- NULL
     }
@@ -1326,16 +1484,40 @@ peek_backtrace_on_error_report <- function() {
   peek_backtrace_on_error_opt("rlang_backtrace_on_error_report") %||% "none"
 }
 
-peek_backtrace_on_error_opt <- function(name) {
-  opt <- peek_option(name)
+peek_backtrace_on_warning_report <- function() {
+  opt <- peek_backtrace_on_error_opt("rlang_backtrace_on_warning_report") %||% "none"
 
-  if (!is_null(opt) && !is_string(opt, backtrace_on_error_opts)) {
-    options(list2("{name}" := NULL))
+  if (is_string(opt, "reminder")) {
+    options(rlang_backtrace_on_warning_report = "none")
     warn(c(
-      sprintf("Invalid %s option.", format_arg(name)),
-      i = "The option was just reset to `NULL`."
+      "`rlang_backtrace_on_warning_report` must be one of `c(\"none\", \"branch\", \"full\")`.",
+      i = "The option was reset to \"none\"."
     ))
-    return(NULL)
+
+    opt <- "none"
+  }
+
+  opt
+}
+
+peek_backtrace_on_error_opt <- function(name) {
+  opt <- peek_option(name)
+
+  if (!is_null(opt)) {
+    if (is_string(opt, "collapse")) {
+      options(list2("{name}" := "none"))
+      deprecate_collapse()
+      return("none")
+    }
+
+    if (!is_string(opt, backtrace_on_error_opts)) {
+      options(list2("{name}" := NULL))
+      warn(c(
+        sprintf("Invalid %s option.", format_arg(name)),
+        i = "The option was just reset to `NULL`."
+      ))
+      return(NULL)
+    }
   }
 
   opt
