@@ -22,7 +22,6 @@
 #'   non-finite values are `NA`, `Inf`, `-Inf` and `NaN`. Setting this
 #'   to something other than `NULL` can be expensive because the whole
 #'   vector needs to be traversed and checked.
-#' @param encoding Defunct as of rlang 0.4.0.
 #' @seealso [bare-type-predicates] [scalar-type-predicates]
 #' @name type-predicates
 NULL
@@ -30,7 +29,7 @@ NULL
 #' @export
 #' @rdname type-predicates
 is_list <- function(x, n = NULL) {
-  .Call(rlang_is_list, x, n)
+  .Call(ffi_is_list, x, n)
 }
 
 parsable_atomic_types <- c("logical", "integer", "double", "complex", "character")
@@ -38,65 +37,74 @@ atomic_types <- c(parsable_atomic_types, "raw")
 #' @export
 #' @rdname type-predicates
 is_atomic <- function(x, n = NULL) {
-  .Call(rlang_is_atomic, x, n)
+  .Call(ffi_is_atomic, x, n)
 }
 #' @export
 #' @rdname type-predicates
 is_vector <- function(x, n = NULL) {
-  .Call(rlang_is_vector, x, n)
+  .Call(ffi_is_vector, x, n)
 }
 
 # Mostly for unit testing
 is_finite <- function(x) {
-  .Call(rlang_is_finite, x)
+  .Call(ffi_is_finite, x)
 }
 
 #' @export
 #' @rdname type-predicates
 is_integer <- function(x, n = NULL) {
-  .Call(rlang_is_integer, x, n)
+  .Call(ffi_is_integer, x, n)
 }
 #' @export
 #' @rdname type-predicates
 is_double <- function(x, n = NULL, finite = NULL) {
-  .Call(rlang_is_double, x, n, finite)
+  .Call(ffi_is_double, x, n, finite)
 }
 #' @export
 #' @rdname type-predicates
-is_character <- function(x, n = NULL, encoding = NULL) {
-  if (!is_null(encoding)) {
-    stop_defunct("The `encoding` argument is deprecated as of rlang 0.3.0.")
+is_complex <- function(x, n = NULL, finite = NULL) {
+  .Call(ffi_is_complex, x, n, finite)
+}
+#' @export
+#' @rdname type-predicates
+is_character <- function(x, n = NULL) {
+  .Call(ffi_is_character, x, n, NULL, NULL)
+}
+is_character2 <- function(x,
+                          n = NULL,
+                          ...,
+                          missing = TRUE,
+                          empty = TRUE) {
+  check_dots_empty0(...)
+
+  # FIXME: Change API at C-level so that `TRUE` means no restriction
+  if (is_true(missing)) {
+    missing <- NULL
   }
-  if (!.Call(rlang_is_character, x, n)) {
-    return(FALSE)
+  if (is_true(empty)) {
+    empty <- NULL
   }
-  if (!is_null(encoding)) {
-    stopifnot(typeof(encoding) == "character")
-    if (!all(Encoding(x) %in% encoding)) {
-      return(FALSE)
-    }
-  }
-  TRUE
+
+  .Call(ffi_is_character, x, n, missing, empty)
 }
 #' @export
 #' @rdname type-predicates
 is_logical <- function(x, n = NULL) {
-  .Call(rlang_is_logical, x, n)
+  .Call(ffi_is_logical, x, n)
 }
 #' @export
 #' @rdname type-predicates
 is_raw <- function(x, n = NULL) {
-  .Call(rlang_is_raw, x, n)
+  .Call(ffi_is_raw, x, n)
 }
 #' @export
 #' @rdname type-predicates
 is_bytes <- is_raw
 
 #' @export
+#' @usage is_null(x)
 #' @rdname type-predicates
-is_null <- function(x) {
-  .Call(rlang_is_null, x)
-}
+is_null <- is.null
 
 #' Scalar type predicates
 #'
@@ -110,7 +118,6 @@ is_null <- function(x) {
 #' type-checking arguments, when your function expects a single string
 #' or a single `TRUE` or `FALSE`.
 #'
-#' @inheritParams type-predicates
 #' @param x object to be tested.
 #' @seealso [type-predicates], [bare-type-predicates]
 #' @name scalar-type-predicates
@@ -119,52 +126,58 @@ NULL
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_list <- function(x) {
-  .Call(rlang_is_list, x, 1L)
+  .Call(ffi_is_list, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_atomic <- function(x) {
-  .Call(rlang_is_atomic, x, 1L)
+  .Call(ffi_is_atomic, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_vector <- function(x) {
-  .Call(rlang_is_vector, x, 1L)
+  .Call(ffi_is_vector, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_integer <- function(x) {
-  .Call(rlang_is_integer, x, 1L)
+  .Call(ffi_is_integer, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_double <- function(x) {
-  .Call(rlang_is_double, x, 1L, NULL)
+  .Call(ffi_is_double, x, 1L, NULL)
 }
 #' @export
 #' @rdname scalar-type-predicates
-is_scalar_character <- function(x, encoding = NULL) {
-  if (!is_null(encoding)) {
-    stop_defunct("The `encoding` argument is deprecated as of rlang 0.3.0.")
-  }
-  is_character(x, encoding = encoding, n = 1L)
+is_scalar_complex <- function(x) {
+  .Call(ffi_is_complex, x, 1L, NULL)
+}
+#' @export
+#' @rdname scalar-type-predicates
+is_scalar_character <- function(x) {
+  is_character(x, n = 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_logical <- function(x) {
-  .Call(rlang_is_logical, x, 1L)
+  .Call(ffi_is_logical, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_raw <- function(x) {
-  .Call(rlang_is_raw, x, 1L)
+  .Call(ffi_is_raw, x, 1L)
 }
 #' @export
 #' @param string A string to compare to `x`. If a character vector,
 #'   returns `TRUE` if at least one element is equal to `x`.
 #' @rdname scalar-type-predicates
 is_string <- function(x, string = NULL) {
-  .Call(rlang_is_string, x, string)
+  .Call(ffi_is_string, x, string, NULL)
+}
+is_string2 <- function(x, string = NULL, ..., empty = NULL) {
+  check_dots_empty0(...)
+  .Call(ffi_is_string, x, string, empty)
 }
 #' @export
 #' @rdname scalar-type-predicates
@@ -216,6 +229,11 @@ is_bare_double <- function(x, n = NULL) {
 }
 #' @export
 #' @rdname bare-type-predicates
+is_bare_complex <- function(x, n = NULL) {
+  !is.object(x) && is_complex(x, n)
+}
+#' @export
+#' @rdname bare-type-predicates
 is_bare_integer <- function(x, n = NULL) {
   !is.object(x) && is_integer(x, n)
 }
@@ -227,11 +245,8 @@ is_bare_numeric <- function(x, n = NULL) {
 }
 #' @export
 #' @rdname bare-type-predicates
-is_bare_character <- function(x, n = NULL, encoding = NULL) {
-  if (!is_null(encoding)) {
-    stop_defunct("The `encoding` argument is deprecated as of rlang 0.3.0.")
-  }
-  !is.object(x) && is_character(x, n, encoding = encoding)
+is_bare_character <- function(x, n = NULL) {
+  !is.object(x) && is_character(x, n)
 }
 #' @export
 #' @rdname bare-type-predicates
@@ -331,7 +346,7 @@ is_false <- function(x) {
 #' is_integerish(10.000001)
 #' is_integerish(TRUE)
 is_integerish <- function(x, n = NULL, finite = NULL) {
-  .Call(rlang_is_integerish, x, n, finite)
+  .Call(ffi_is_integerish, x, n, finite)
 }
 #' @rdname is_integerish
 #' @export
@@ -341,12 +356,12 @@ is_bare_integerish <- function(x, n = NULL, finite = NULL) {
 #' @rdname is_integerish
 #' @export
 is_scalar_integerish <- function(x, finite = NULL) {
-  .Call(rlang_is_integerish, x, 1L, finite)
+  .Call(ffi_is_integerish, x, 1L, finite)
 }
 
 type_of_ <- function(x) {
   type <- typeof(x)
-  if (is_formulaish(x)) {
+  if (is_formula(x)) {
     if (identical(node_car(x), colon_equals_sym)) {
       "definition"
     } else {
@@ -361,26 +376,6 @@ type_of_ <- function(x) {
   }
 }
 
-#' Format a type for error messages
-#'
-#' @section Life cycle:
-#'
-#' * `friendly_type()` is experimental.
-#'
-#' @param type A type as returned by [typeof()].
-#' @return A string of the prettified type, qualified with an
-#'   indefinite article.
-#' @export
-#' @keywords internal
-#' @examples
-#' friendly_type("logical")
-#' friendly_type("integer")
-#' friendly_type("string")
-#' @export
-friendly_type <- function(type) {
-  as_friendly_type(type) %||% type
-}
-
 #' Is an object copyable?
 #'
 #' When an object is modified, R generally copies it (sometimes
@@ -468,7 +463,7 @@ is_equal <- function(x, y) {
 #' vec[[1]] <- 100
 #' is_reference(copy, vec)
 is_reference <- function(x, y) {
-  .Call(rlang_is_reference, x, y)
+  .Call(ffi_is_reference, x, y)
 }
 
 
@@ -478,18 +473,25 @@ rlang_type_sum <- function(x) {
   if (is_installed("pillar")) {
     pillar::type_sum(x)
   } else {
-    UseMethod("type_sum")
+    UseMethod("rlang_type_sum")
   }
 }
 
-type_sum.ordered <- function(x) "ord"
-type_sum.factor <- function(x) "fct"
-type_sum.POSIXct <- function(x) "dttm"
-type_sum.difftime <- function(x) "time"
-type_sum.Date <- function(x) "date"
-type_sum.data.frame <- function(x) class(x)[[1]]
+#' @export
+rlang_type_sum.ordered <- function(x) "ord"
+#' @export
+rlang_type_sum.factor <- function(x) "fct"
+#' @export
+rlang_type_sum.POSIXct <- function(x) "dttm"
+#' @export
+rlang_type_sum.difftime <- function(x) "time"
+#' @export
+rlang_type_sum.Date <- function(x) "date"
+#' @export
+rlang_type_sum.data.frame <- function(x) class(x)[[1]]
 
-type_sum.default <- function(x) {
+#' @export
+rlang_type_sum.default <- function(x) {
   if (!is.object(x)) {
     switch(typeof(x),
       logical = "lgl",
