@@ -29,6 +29,8 @@ test_that("error if dots named", {
     check_dots_unnamed()
   }
 
+  expect_null(f(1))
+
   expect_error(f(xyz = 1), NA)
   expect_error(f(1, 2, 3), NA)
   expect_error(f(1, 2, 3, xyz = 4), NA)
@@ -92,16 +94,15 @@ test_that("can supply `error` handler", {
 
   f <- function(...) check_dots_empty(error = hnd)
   expect_silent(f())
-  expect_warning(f(foo), class = "rlib_error_dots_nonempty")
+  expect_error(f(foo), class = "rlib_error_dots_nonempty")
 
   f <- function(...) check_dots_used(error = hnd)
   expect_silent(f())
-  expect_warning(f(foo), class = "rlib_error_dots_unused")
-
+  expect_error(f(foo), class = "rlib_error_dots_unused")
 
   f <- function(...) check_dots_unnamed(error = hnd)
   expect_silent(f(foo))
-  expect_warning(f(foo = foo), class = "rlib_error_dots_named")
+  expect_error(f(foo = foo), class = "rlib_error_dots_named")
 })
 
 test_that("expression contents are mentioned", {
