@@ -1,8 +1,6 @@
-context("node")
-
 test_that("node() creates a pairlist node", {
   x <- new_node("foo", "bar")
-  expect_is(x, "pairlist")
+  expect_type(x, "pairlist")
   expect_identical(node_car(x), "foo")
   expect_identical(node_cdr(x), "bar")
 })
@@ -47,34 +45,24 @@ test_that("node_tree_clone() clones all nodes", {
   clone <- node_tree_clone(x)
 
   # Outer vector
-  expect_false(sexp_address(x) == sexp_address(clone))
+  expect_false(obj_address(x) == obj_address(clone))
 
   # Outer node list
-  expect_true(sexp_address(node_car(x)) == sexp_address(node_car(clone)))
+  expect_true(obj_address(node_car(x)) == obj_address(node_car(clone)))
 
   cdr <- node_cdr(x)
   clone_cdr <- node_cdr(clone)
-  expect_false(sexp_address(cdr) == sexp_address(clone_cdr))
+  expect_false(obj_address(cdr) == obj_address(clone_cdr))
 
   # Inner node list
   cadr <- node_car(cdr)
   clone_cadr <- node_car(clone_cdr)
-  expect_false(sexp_address(cadr) == sexp_address(clone_cadr))
+  expect_false(obj_address(cadr) == obj_address(clone_cadr))
 
   # Inner vector
   caadr <- node_car(cadr)
   clone_caadr <- node_car(clone_cadr)
-  expect_true(sexp_address(caadr) == sexp_address(clone_caadr))
-})
-
-test_that("as_pairlist() converts to pairlist", {
-  expect_identical(as_pairlist(letters), as.pairlist(letters))
-  expect_error(as_pairlist(quote(foo)), "Can't convert a symbol to a pairlist node")
-
-  expect_identical(as_pairlist(NULL), NULL)
-
-  x <- pairlist(1, 2)
-  expect_identical(as_pairlist(x), x)
+  expect_true(obj_address(caadr) == obj_address(clone_caadr))
 })
 
 test_that("pairlist predicates detect pairlists", {
@@ -95,10 +83,10 @@ test_that("pairlist2() converts to pairlist", {
   expect_identical_(pairlist2(1, !!!mtcars[1:2], 4), pairlist(1, mpg = mtcars$mpg, cyl = mtcars$cyl, 4))
 
   local_bindings(.env = global_env(),
-    as.list.rlang_foobar = function(x) list("foo", "bar")
+    `[[.rlang_foobar` = function(x, i) "foo"
   )
   foobar <- structure(NA, class = "rlang_foobar")
-  expect_identical_(pairlist2(1, !!!foobar, 4), pairlist(1, "foo", "bar", 4))
+  expect_identical_(pairlist2(1, !!!foobar, 4), pairlist(1, "foo", 4))
 })
 
 test_that("pairlist2() duplicates spliced pairlists", {
